//+------------------------------------------------------------------+
//|                                                          GIM.mq4 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum Type_mode
  {
   BUY,
   SELL,
  };

input int     Magic=16384;       //   
input Type_mode Type=BUY; //  
input double  lots       = 0.01;   // 
input int     stop_loss      = 10; //    
input int     take_profit    = 20; //    
extern int    Slippage=300;        //     

int error=0;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   NevBar();
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   if(NevBar() && Total_orders(Type)==0)OrderOpen(Type);
   if(Profit_order(EMPTY_VALUE)>=take_profit)Order_Close(EMPTY_VALUE);
   if(Profit_order(EMPTY_VALUE)<=-stop_loss)Order_Close(EMPTY_VALUE);

  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
bool NevBar()
  {
   static datetime StatTime;

   if(StatTime!=iTime(NULL,PERIOD_CURRENT,0))
     {
      StatTime=iTime(NULL,PERIOD_CURRENT,0);
      return(true);
     }
   return(false);
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
int Total_orders(int type)
  {
   int n=0;
   int total=OrdersTotal();

   for(int i=0; i<total; i++)
     {
      if(OrderSelect(i,SELECT_BY_POS))
        {
         if(OrderType()!=type && type!=EMPTY_VALUE)continue;
         if(OrderMagicNumber()!=Magic)continue;
         if(OrderSymbol()!=_Symbol)continue;
         n++;
        }

     }
   return(n);
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
double Profit_order(int type)
  {
   double profit=0;
   int total=OrdersTotal();

   for(int i=0; i<total; i++)
     {
      if(OrderSelect(i,SELECT_BY_POS))
        {
         if(OrderType()!=type && type!=EMPTY_VALUE)continue;
         if(OrderMagicNumber()!=Magic)continue;
         if(OrderSymbol()!=_Symbol)continue;
         profit+=OrderProfit()+OrderSwap()+OrderCommission();
        }

     }
   return(profit);
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
void OrderOpen(int type)
  {
   double price=0.0;
   double lot;

   if(error==2)return;

   lot=LotNormalize(lots,_Symbol);
   if(lot < MarketInfo(_Symbol,MODE_MINLOT))lot = MarketInfo(_Symbol,MODE_MINLOT);
   if(lot > MarketInfo(_Symbol,MODE_MAXLOT))lot = MarketInfo(_Symbol,MODE_MAXLOT);

   do
     {
      if(type==OP_BUY)
        {
         RefreshRates();
         price=NormalizeDouble(Ask,_Digits);
         if(AccountFreeMarginCheck(_Symbol,type,lot)<=0)return;
        }
      if(type==OP_SELL)
        {
         RefreshRates();
         price=NormalizeDouble(Bid,_Digits);
         if(AccountFreeMarginCheck(_Symbol,type,lot)<=0)return;
        }

      int Ticket=OrderSend(_Symbol,type,lot,price,Slippage,0,0,NULL,Magic);
      if(Ticket>0)
        {
         error=Fun_Error(GetLastError());
         Alert("  ",StrToType(type)," ",Ticket);
        }
      else error=Fun_Error(GetLastError());
     }
   while(error>0 && !IsStopped());
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
void Order_Close(int type)
  {
   double price=0.0;
   int total=OrdersTotal();
   if(error==2)return;

   for(int i=total-1; i>=0; i--)
     {
      if(OrderSelect(i,SELECT_BY_POS))
        {
         if(OrderType()!=type && type!=EMPTY_VALUE)continue;
         if(OrderMagicNumber()!=Magic)continue;
         if(OrderSymbol()!=_Symbol)continue;
         do
           {
            if(OrderType()==OP_BUY)
              {
               RefreshRates();
               price=NormalizeDouble(Bid,_Digits);
              }
            if(OrderType()==OP_SELL)
              {
               RefreshRates();
               price=NormalizeDouble(Ask,_Digits);
              }
            int res=OrderClose(OrderTicket(),OrderLots(),price,Slippage,clrBlue);
            if(res)
              {
               Alert("  ",StrToType(OrderType())," ",OrderTicket());
               error=Fun_Error(GetLastError());
              }
            else error=Fun_Error(GetLastError());

           }
         while(error>0 && !IsStopped());

        }
     }

  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
double LotNormalize(double lot_,string symbol)
  {
   double minlot=MarketInfo(symbol,MODE_MINLOT);

   if(minlot==0.001)      return(NormalizeDouble(lot_,3));
   else if(minlot==0.01)  return(NormalizeDouble(lot_,2));
   else if(minlot==0.1)   return(NormalizeDouble(lot_,1));

   return(NormalizeDouble(lot_,0));
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
string StrToType(int type)
  {
   if(type==OP_BUY)return("Buy");
   if(type==OP_SELL)return("Sell");
   return(NULL);
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
int Fun_Error(int Error)
  {
   switch(Error)
     {
      case 0:return(0);
      case 4:Alert("  .   ...");
      Sleep(3000);
      return(1);
      case 135:Alert(" .   ...");
      RefreshRates();
      return(1);
      case 136:Alert(" .   ...");
      while(RefreshRates()==false && !IsStopped())
         Sleep(1);
      return(1);
      case 137:Alert(" .   ...");
      Sleep(3000);
      return(1);
      case 146:Alert("  .   ...");
      Sleep(500);
      return(1);

      // 

      case 2:Alert(" .");
      return(2);
      case 5:Alert("  .");
      //Work=false;
      return(2);
      case 64:Alert(" .");
      //Work=false;
      return(2);
      case 133:Alert(" .");
      return(2);
      case 134:Alert("    .");
      return(2);
      default:Alert("  -  ",Error);
      return(2);
     }
  }
//+------------------------------------------------------------------+
